#include "TArtNEBULAParameters.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtCore.h"
#include "TArtStoreManager.h"
#include "TArtMap.h"

#include <TDOMParser.h>
#include <TXMLNode.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <vector>

//__________________________________________________________
TArtNEBULAParameters::TArtNEBULAParameters(const char* name, const char* title)
  : TNamed(name, title)
{
  TArtCore::Info(__FILE__,"Creating NEBULA setup...");

  pumap = new TArtNEBULAPlaParaMap;
  pdmap = new TArtNEBULAPlaParaMap;

  fStoreManager = TArtStoreManager::Instance();
  fStoreManager->AddParameters(this);
}

//__________________________________________________________
TArtNEBULAParameters::~TArtNEBULAParameters() 
{
  {
    TArtNEBULAPlaParaMap::iterator itr = pumap->begin();
    std::vector<const TArtNEBULAPlaPara*> pvec;
    while(itr != pumap->end()){
      std::vector<const TArtNEBULAPlaPara*>::iterator it_pvec = pvec.begin();
      while(it_pvec != pvec.end()){
	if(*it_pvec == itr->second){
	  pvec.push_back(itr->second);
	  delete itr->second;
	  break;
	}
	++it_pvec;
      }
      itr->second = 0;
      ++itr;
    }
  }
  {
    TArtNEBULAPlaParaMap::iterator itr = pdmap->begin();
    std::vector<const TArtNEBULAPlaPara*> pvec;
    while(itr != pdmap->end()){
      std::vector<const TArtNEBULAPlaPara*>::iterator it_pvec = pvec.begin();
      while(it_pvec != pvec.end()){
	if(*it_pvec == itr->second){
	  pvec.push_back(itr->second);
	  delete itr->second;
	  break;
	}
	++it_pvec;
      }
      itr->second = 0;
      ++itr;
    }
  }
}

//__________________________________________________________
Bool_t TArtNEBULAParameters::LoadParameter(const char *xmlfile)
{
  TArtCore::Info(__FILE__,"Load parameter from %s", xmlfile);
  TDOMParser* domParser = new TDOMParser();
  domParser->SetValidate(false);
  Int_t parsecode = domParser->ParseFile(xmlfile);
  if(parsecode < 0){
    cerr << domParser->GetParseCodeMessage(parsecode) << endl;
    return false;
  }
  TXMLNode* node = domParser->GetXMLDocument()->GetRootNode();
  ParseParaList(node->GetChildren());
  return true;
}

//__________________________________________________________
void TArtNEBULAParameters::ParseParaList(TXMLNode *node)
{
  for(; node; node = node->GetNextNode()){
    if(node->GetNodeType() == TXMLNode::kXMLElementNode){ // Element Node
      if(strcmp(node->GetNodeName(), "NEBULA") == 0){
	ParseNEBULAPlaPara(node->GetChildren());
      }
    }
  }
  return;
}

//__________________________________________________________
TArtNEBULAPlaPara *TArtNEBULAParameters::ParseNEBULAPlaPara(TXMLNode *node)
{
  TArtNEBULAPlaPara* para = new TArtNEBULAPlaPara;
  Int_t tu_geo = -1, tu_ch = -1, qu_geo = -1, qu_ch = -1;
  Int_t td_geo = -1, td_ch = -1, qd_geo = -1, qd_ch = -1;

  for(; node; node = node->GetNextNode()){
    if(node->GetNodeType() == TXMLNode::kXMLElementNode){ // Element Node
      if(strcmp(node->GetNodeName(), "ID") == 0){ 
	Int_t id = atoi(node->GetText());
	para->SetID(id);
      }else if(strcmp(node->GetNodeName(), "NAME") == 0){
	TString name = node->GetText();
	para->SetDetectorName(name);
      }else if(strcmp(node->GetNodeName(), "FPl") == 0){
	Int_t fpl = atoi(node->GetText());
	para->SetFPl(fpl);
      }else if(strcmp(node->GetNodeName(), "Layer") == 0){
	Int_t layer = atoi(node->GetText());
	para->SetLayer(layer);
      }else if(strcmp(node->GetNodeName(), "SubLayer") == 0){
	Int_t sublayer = atoi(node->GetText());
	para->SetSubLayer(sublayer);
      }else if(strcmp(node->GetNodeName(), "PosX") == 0){
	Double_t posx = atof(node->GetText());
	para->SetDetPos(posx, 0);
      }else if(strcmp(node->GetNodeName(), "PosY") == 0){
	Double_t posy = atof(node->GetText());
	para->SetDetPos(posy, 1);
      }else if(strcmp(node->GetNodeName(), "PosZ") == 0){
	Double_t posz = atof(node->GetText());
	para->SetDetPos(posz, 2);
      }else if(strcmp(node->GetNodeName(), "TUCal") == 0){
	Double_t tucal = atof(node->GetText());
	para->SetTUCal(tucal);
      }else if(strcmp(node->GetNodeName(), "TUOff") == 0){
	Double_t tuoff = atof(node->GetText());
	para->SetTUOff(tuoff);
      }else if(strcmp(node->GetNodeName(), "TUSlw") == 0){
	Double_t tuslw = atof(node->GetText());
	para->SetTUSlw(tuslw);
      }else if(strcmp(node->GetNodeName(), "QUCal") == 0){
	Double_t qucal = atof(node->GetText());
	para->SetQUCal(qucal);
      }else if(strcmp(node->GetNodeName(), "QUPed") == 0){
	Double_t quped = atof(node->GetText());
	para->SetQUPed(quped);
      }else if(strcmp(node->GetNodeName(), "TDCal") == 0){
	Double_t tdcal = atof(node->GetText());
	para->SetTDCal(tdcal);
      }else if(strcmp(node->GetNodeName(), "TDOff") == 0){
	Double_t tdoff = atof(node->GetText());
	para->SetTDOff(tdoff);
      }else if(strcmp(node->GetNodeName(), "TDSlw") == 0){
	Double_t tdslw = atof(node->GetText());
	para->SetTDSlw(tdslw);
      }else if(strcmp(node->GetNodeName(), "QDCal") == 0){
	Double_t qdcal = atof(node->GetText());
	para->SetQDCal(qdcal);
      }else if(strcmp(node->GetNodeName(), "QDPed") == 0){
	Double_t qdped = atof(node->GetText());
	para->SetQDPed(qdped);
      }else if(strcmp(node->GetNodeName(), "DTCal") == 0){
	Double_t dtcal = atof(node->GetText());
	para->SetDTCal(dtcal);
      }else if(strcmp(node->GetNodeName(), "DTOff") == 0){
	Double_t dtoff = atof(node->GetText());
	para->SetDTOff(dtoff);
      }else if(strcmp(node->GetNodeName(), "TAveOff") == 0){
	Double_t taveoff = atof(node->GetText());
	para->SetTAveOff(taveoff);
      }else if(strcmp(node->GetNodeName(), "Ignore") == 0){
	Int_t ignore = atoi(node->GetText());
	para->SetIgnore(ignore);
      }else if(strcmp(node->GetNodeName(), "tu_geo") == 0){
	tu_geo = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "tu_ch") == 0){
	tu_ch = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "qu_geo") == 0){
	qu_geo = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "qu_ch") == 0){
	qu_ch = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "td_geo") == 0){
	td_geo = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "td_ch") == 0){
	td_ch = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "qd_geo") == 0){
	qd_geo = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "qd_ch") == 0){
	qd_ch = atoi(node->GetText());
      }
    }
  }
  
  para->SetMapTU(tu_geo, tu_ch);
  para->SetMapQU(qu_geo, qu_ch);
  para->SetMapTD(td_geo, td_ch);
  para->SetMapQD(qd_geo, qd_ch);
  pumap->insert(TArtNEBULAPlaParaPair(para->GetTUMap(), para));
  pumap->insert(TArtNEBULAPlaParaPair(para->GetQUMap(), para));
  pdmap->insert(TArtNEBULAPlaParaPair(para->GetTDMap(), para));
  pdmap->insert(TArtNEBULAPlaParaPair(para->GetQDMap(), para));

  fIDPlaParaMap.insert(std::pair<int,TArtNEBULAPlaPara*>(para->GetID(), para)); //optimization

  return para;
}

const TArtNEBULAPlaPara* TArtNEBULAParameters::FindNEBULAPlaParaU(const TArtRIDFMap &rmap) const
{
  TArtNEBULAPlaParaMap::const_iterator itr = pumap->find(rmap);
  if(itr != pumap->end()) return itr->second;
  else return 0;
}

const TArtNEBULAPlaPara* TArtNEBULAParameters::FindNEBULAPlaParaD(const TArtRIDFMap &rmap) const
{
  TArtNEBULAPlaParaMap::const_iterator itr = pdmap->find(rmap);
  if(itr != pdmap->end()) return itr->second;
  else return 0;
}

const TArtNEBULAPlaPara* TArtNEBULAParameters::FindNEBULAPlaPara(int id) const
{
//  TArtNEBULAPlaParaMap::const_iterator itu = pumap->begin();
//  while(itu != pumap->end()){
//    const TArtNEBULAPlaPara* para = itu->second;
//    if(id == para->GetID()) return para;
//    ++itu;
//  }
//  TArtNEBULAPlaParaMap::const_iterator itd = pdmap->begin();
//  while(itd != pdmap->end()){
//    const TArtNEBULAPlaPara* para = itd->second;
//    if(id == para->GetID()) return para;
//    ++itd;
//  }
//  return 0;

  //optimization
  std::map<int,TArtNEBULAPlaPara*>::const_iterator it = fIDPlaParaMap.find(id);
  if(it != fIDPlaParaMap.end()) return it->second;
  else return 0;
}

